;; -*- lexical-binding: t; -*-

;; Unsorted original configs.

;;;; Packages.
(require 'package)
;; Make sure to use HTTPS.
(setq package-archives
      ;; '("melpa-stable" . "http://melpa-stable.org/packages/")
      '(("gnu" . "https://elpa.gnu.org/packages/")
        ("melpa" . "https://melpa.org/packages/")))
(package-initialize)

;; Check TLS (https://glyph.twistedmatrix.com/2015/11/editor-malware.html)
(setq tls-checktrust t)
(setq gnutls-verify-error t)

;;;; Basic configs.
(require 'cl-lib)
(require 'subr-x)

;; Define C-z as a new prefix key; because it usually is a standard Emacs
;; keybinding, no mode uses it, so we can have it for ourselves without fear
;; of conflicts. (Also, C-z is very convenient to type.) Suspending can still
;; be done with C-x C-z.
(define-prefix-command 'ctl-z-map)
(global-set-key (kbd "C-z") 'ctl-z-map)

;;;;;; Backups.

(let ((backup-dir "~/.cache/emacs/backups")
      (autosave-dir "~/.cache/emacs/autosave/"))
  ;; Save backup files in ~/backups instead of the file's directory.
  (mkdir backup-dir t)
  (setq backup-directory-alist `(("." . ,backup-dir)))
  ;; Likewise for autosave files.
  (mkdir autosave-dir t)
  (setq auto-save-file-name-transforms `((".*" ,autosave-dir t)))
  (setq auto-save-list-file-prefix "~/.cache/emacs/auto-save-list/.saves-")
  )

;;;;;; Enabled commands.
(put 'narrow-to-region 'disabled nil)
(put 'upcase-region 'disabled nil)
(put 'downcase-region 'disabled nil)


;;;;; Visit config file.

(defun elmord-visit-init-file ()
  (interactive)
  (find-file "~/.emacs.d/init.d"))

(define-key ctl-z-map (kbd "e") 'elmord-visit-init-file)


;;;; Appearance and stuff.

(add-to-list 'default-frame-alist '(foreground-color . "white"))
(add-to-list 'default-frame-alist '(background-color . "black"))
(tool-bar-mode 0)
(menu-bar-mode 0)
(blink-cursor-mode 0)
(fringe-mode 1)
(scroll-bar-mode 0)
(column-number-mode 1)
(size-indication-mode 1)

(setq history-length 500)
(setq savehist-file "~/.cache/emacs/history")
(savehist-mode 1)

;; Show clock in mode line, but only in full-screen mode.
(setq display-time-default-load-average nil)
(setq display-time-mail-file 'none)
(setq display-time-24hr-format t)



(setq frame-title-format '("%b (" default-directory ")"))

;;;; Miscellaneous.

;; Cache GPG passphrases (questionable! should use gpg-agent instead)
(setq epa-file-cache-passphrase-for-symmetric-encryption t)

(global-set-key (kbd "M-g M-f") 'find-file-at-point)


;; Remove special action of ? in minibuffer completion.
(defun elmord-delq-alist (key alist)
  (let ((binding (assq key alist)))
    (when binding
      (delq binding alist))))

(setq minibuffer-local-completion-map
      (elmord-delq-alist ?? minibuffer-local-completion-map))

;; Persistent scratch buffer (is this a good idea?)
;; Vaguely based on http://dorophone.blogspot.com.br/2011/11/how-to-make-emacs-scratch-buffer.html

(defvar elmord-scratch-file "~/.emacs.d/scratch.org")

(defun elmord-load-scratch-file ()
  (with-current-buffer (get-buffer "*scratch*")
    (when (file-exists-p elmord-scratch-file)
      (delete-region (point-min) (point-max))
      (insert-file elmord-scratch-file)
      (org-mode))
    ;; Associate buffer with file, but restore original buffer name.
    (set-visited-file-name elmord-scratch-file)
    (set-buffer-modified-p nil)
    (cd "~")
    (rename-buffer "*scratch*")))

(elmord-load-scratch-file) ;; load on init

;; Misc.
(ido-mode 'buffers)
(setq ido-default-buffer-method 'selected-window)

;; PDF Tools and PDF stuff.

(when (package-installed-p 'pdf-tools)
  (require 'pdf-tools)
  (pdf-tools-install)
  (setq pdf-view-midnight-colors '("#ffffff" . "#000000")))

(add-hook 'pdf-view-mode-hook 'pdf-view-midnight-minor-mode)

;;;; Input methods.
;; Work around buggy 'a' in Tibetan input method. This effectively
;; turns 'a' into a null key which just breaks consonant stacking.
(eval-after-load 'tibetan
  (defun elmord-tibetan-fixup-hook ()
    (setq tibetan-vowel-transcription-alist
          (remq (assoc "a" tibetan-vowel-transcription-alist)
                tibetan-vowel-transcription-alist))))


;;;; Org mode.
(require 'org)
(setq org-startup-truncated nil)

(setq org-default-notes-file "~/org/notes.org")
(define-key global-map "\C-cc" 'org-capture)


;;;; Utilities.

(defun elmord-decode-event (event)
  "Example: (elmord-decode-event (kbd \"s-a\"))"
  (if (or (vectorp event) (stringp event))
      (map 'vector 'elmord-decode-event event)
    (let ((modifiers (event-modifiers event))
          (base (event-basic-type event)))
      (cons modifiers (if (characterp base)
                          (string base)
                        base)))))


(global-set-key (kbd "C-x g") 'magit-status)

(add-to-list 'auto-mode-alist '("\\.[hH][tT][mM][lL]\\'" . web-mode))

(server-start)
